@@ -14,7 +14,8 @@
 # sudo apt-get install gcc-multilib g++-multilib
 # $ AUTOCONF_FLAGS="--host=i686-linux-gnu CFLAGS=-m32 CXXFLAGS=-m32 LDFLAGS=-m32" ./contrib/make_libsecp256k1.sh
 
-LIBSECP_VERSION="1253a27756540d2ca526b2061d98d54868e9177c"
+LIBSECP_VERSION="346a053d4c442e08191f075c3932d03140579d47"
+# ^ tag "v0.3.1"
 
 set -e
 
@@ -42,7 +43,6 @@ info "Building $pkgname..."
     git checkout "${LIBSECP_VERSION}^{commit}"
 
     if ! [ -x configure ] ; then
-        echo "libsecp256k1_la_LDFLAGS = -no-undefined" >> Makefile.am
         echo "LDFLAGS = -no-undefined" >> Makefile.am
         ./autogen.sh || fail "Could not run autogen for $pkgname. Please make sure you have automake and libtool installed, and try again."
     fi
@@ -59,13 +59,13 @@ info "Building $pkgname..."
             --disable-static \
             --enable-shared || fail "Could not configure $pkgname. Please make sure you have a C compiler installed and try again."
     fi
-    make -j4 || fail "Could not build $pkgname"
+    make "-j$CPU_COUNT" || fail "Could not build $pkgname"
     make install || fail "Could not install $pkgname"
     . "$here/$pkgname/dist/lib/libsecp256k1.la"
     host_strip "$here/$pkgname/dist/lib/$dlname"
     cp -fpv "$here/$pkgname/dist/lib/$dlname" "$PROJECT_ROOT/electrum" || fail "Could not copy the $pkgname binary to its destination"
     info "$dlname has been placed in the inner 'electrum' folder."
     if [ -n "$DLL_TARGET_DIR" ] ; then
-        cp -fpv "$here/$pkgname/dist/lib/$dlname" "$DLL_TARGET_DIR" || fail "Could not copy the $pkgname binary to DLL_TARGET_DIR"
+        cp -fpv "$here/$pkgname/dist/lib/$dlname" "$DLL_TARGET_DIR/" || fail "Could not copy the $pkgname binary to DLL_TARGET_DIR"
     fi
 )
