@@ -1,11 +1,12 @@
 from decimal import Decimal
 from typing import Optional
+from functools import partial
 
 from PyQt5.QtCore import pyqtProperty, pyqtSignal, pyqtSlot, QObject
 
 from electrum.logging import get_logger
 from electrum.i18n import _
-from electrum.transaction import PartialTxOutput, PartialTransaction
+from electrum.transaction import PartialTxOutput, PartialTransaction, Transaction
 from electrum.util import NotEnoughFunds, profiler
 from electrum.wallet import CannotBumpFee, CannotDoubleSpendTx, CannotCPFP
 from electrum.network import NetworkException
@@ -220,7 +221,7 @@ class TxFeeSlider(FeeSlider):
 class QETxFinalizer(TxFeeSlider):
     _logger = get_logger(__name__)
 
-    finishedSave = pyqtSignal([str], arguments=['txid'])
+    finished = pyqtSignal([bool, bool, bool], arguments=['signed', 'saved', 'complete'])
 
     def __init__(self, parent=None, *, make_tx=None, accept=None):
         super().__init__(parent)
@@ -301,6 +302,10 @@ class QETxFinalizer(TxFeeSlider):
         return tx
 
     def update(self):
+        if not self._wallet:
+            self._logger.debug('wallet not set, ignoring update()')
+            return
+
         try:
             # make unsigned transaction
             tx = self.make_tx(amount = '!' if self._amount.isMax else self._amount.satsInt)
@@ -342,13 +347,17 @@ class QETxFinalizer(TxFeeSlider):
         self.validChanged.emit()
 
     @pyqtSlot()
-    def save(self):
+    def saveOrShow(self):
         if not self._valid or not self._tx:
             self._logger.debug('no valid tx')
             return
 
-        if self._wallet.save_tx(self._tx):
-            self.finishedSave.emit(self._tx.txid())
+        saved = False
+        if self._tx.txid():
+            if self._wallet.save_tx(self._tx):
+                saved = True
+
+        self.finished.emit(False, saved, self._tx.is_complete())
 
     @pyqtSlot()
     def signAndSend(self):
@@ -360,38 +369,36 @@ class QETxFinalizer(TxFeeSlider):
             self.f_accept(self._tx)
             return
 
-        self._wallet.sign(self._tx, broadcast=True)
+        self._wallet.sign(self._tx,
+            broadcast=True,
+            on_success=partial(self.on_signed_tx, False)
+        )
 
     @pyqtSlot()
-    def signAndSave(self):
+    def sign(self):
         if not self._valid or not self._tx:
             self._logger.error('no valid tx')
             return
 
-        # TODO: f_accept handler not used
-        # if self.f_accept:
-        #     self.f_accept(self._tx)
-        #     return
+        self._wallet.sign(self._tx,
+            broadcast=False,
+            on_success=partial(self.on_signed_tx, True)
+        )
 
-        try:
-            self._wallet.transactionSigned.disconnect(self.onSigned)
-        except:
-            pass
-        self._wallet.transactionSigned.connect(self.onSigned)
-        self._wallet.sign(self._tx)
+    def on_signed_tx(self, save: bool, tx: Transaction):
+        self._logger.debug('on_signed_tx')
+        saved = False
+        if save and self._tx.txid():
+            if self._wallet.save_tx(self._tx):
+                saved = True
+            else:
+                self._logger.error('Could not save tx')
+        self.finished.emit(True, saved, tx.is_complete())
 
-    @pyqtSlot(str)
-    def onSigned(self, txid):
-        if txid != self._tx.txid():
-            return
-
-        self._logger.debug('onSigned')
-        self._wallet.transactionSigned.disconnect(self.onSigned)
-
-        if not self._wallet.save_tx(self._tx):
-            self._logger.error('Could not save tx')
-        else:
-            self.finishedSave.emit(self._tx.txid())
+    @pyqtSlot(result='QVariantList')
+    def getSerializedTx(self):
+        txqr = self._tx.to_qr_data()
+        return [str(self._tx), txqr[0], txqr[1]]
 
 # mixin for watching an existing TX based on its txid for verified event
 # requires self._wallet to contain a QEWallet instance
